import * as utility from "./toolUtil.js";

let lsbSize;
let isLsbSizeCalculated = false;
// ---------- Controls ----------

export function updateOutputCode(args) {
  isLsbSizeCalculated = false;
  let module = args.parameterConfigData[args.selectedDevice].calculationModule;
  if (Object.keys(module).length > 0) {
    if (module.updateControlInfo) {
      module.updateControlInfo(
        args,
        args.parameterConfigData[args.selectedDevice].parameters,
        args.parameterConfigData[args.selectedDevice].controls.COMMON,
        args.parameterConfigData[args.selectedDevice].controls.codeToVoltage,
        args.parameterConfigData[args.selectedDevice].controls.voltageToCode,
      );
    }
  }
  const isHexUnit = utility.isHexUnit(args.output_code.unit.value);
  let decimalCode = utility.getDecNum(
    args.output_code.value,
    parseInt(
      args.parameterConfigData[args.selectedDevice].parameters.resolution,
    ),
    args.twos_complement.value,
  );

  const maxCode = this.getMaxCode(args);
  const minCode = this.getMinCode(args);

  return {
    controlType: isHexUnit ? "hex-input" : "input-spinner",

    value: isHexUnit
      ? utility.decToHexStr(
          decimalCode,
          parseInt(
            args.parameterConfigData[args.selectedDevice].parameters.resolution,
          ),
          args.twos_complement.value,
        )
      : decimalCode,

    size:
      parseInt(
        args.parameterConfigData[args.selectedDevice].parameters.resolution,
      ) / 4,

    isTwosComplement: args.twos_complement.value,

    invalid: {
      ...args.output_code.invalid,
      status: !this.isValidOutputCode(args),
    },

    minValue: isHexUnit
      ? utility.decToHexStr(
          minCode,
          parseInt(
            args.parameterConfigData[args.selectedDevice].parameters.resolution,
          ),
          args.twos_complement.value,
        )
      : minCode,

    maxValue: isHexUnit
      ? utility.decToHexStr(
          maxCode,
          parseInt(
            args.parameterConfigData[args.selectedDevice].parameters.resolution,
          ),
          args.twos_complement.value,
        )
      : maxCode,
  };
}

export function updateVoltageRtiInput(args) {
  const inputRange = (
    this.getInputRange(args) /
    utility.getMultiplierForUnit(args.voltage_rti.unit.value)
  ).toFixed(2);

  return {
    minValue: -parseFloat(inputRange),
    maxValue: parseFloat(inputRange),
  };
}

// ---------- Outputs ----------

export function computeAdcInputRangeLsbSize(args) {
  return {
    value: this.getAdcInputRangeLsbSizeTable(
      this.getInputRange(args),
      isLsbSizeCalculated ? lsbSize : this.getLsbSizeValue(args),
    ),
  };
}

export function computeOutputCodeOutput(args) {
  let decimalCode = this.getOutputCodeOutput(args);

  return {
    value: utility.isHexUnit(args.output_code_output.unit.value)
      ? utility.decToHexStr(
          decimalCode,
          parseInt(
            args.parameterConfigData[args.selectedDevice].parameters.resolution,
          ),
          args.twos_complement.value,
        )
      : decimalCode,

    error: {
      ...args.output_code_output.error,
      status: this.getVoltageRangeStr(args) !== "-",
    },
  };
}

export function computeVoltageRtiOutput(args) {
  const validOutputCode = this.isValidOutputCode(args);

  return {
    value: validOutputCode
      ? this.getVoltageRtiOutput(args).toFixed(
          args.voltage_rti_output.precision,
        )
      : 0,

    error: {
      ...args.voltage_rti_output.error,
      status: !validOutputCode,
    },
  };
}

// ---------- Helpers ----------

export function getAdcInputRangeLsbSizeTable(adcInputRange, lsbSize) {
  let table = [];

  table.push(
    utility.getTableRow(
      [
        {
          value: "ADC Input Range & Least Significant Bit (LSB) Size",
          colspan: 2,
        },
      ],
      "head",
    ),
  );

  table.push(
    utility.getTableRow(
      [
        {
          value: "Input Range",
        },
        {
          value: "LSB size",
        },
      ],
      "head",
    ),
  );

  table.push(
    utility.getTableRow([
      {
        value: "+/-" + adcInputRange.toFixed(2) + " V",
      },
      {
        value: (lsbSize * 1000000).toFixed(3) + " µV",
      },
    ]),
  );

  return table;
}

export function getOutputCodeOutput(args) {
  let voltageOutOfRange = this.getVoltageRangeStr(args);

  if (voltageOutOfRange === "POSITIVE") {
    return this.getMaxCode(args);
  } else if (voltageOutOfRange === "NEGATIVE") {
    return this.getMinCode(args);
  } else {
    return Math.round(
      this.getInputVoltageRti(args) /
        (isLsbSizeCalculated ? lsbSize : this.getLsbSizeValue(args)),
    );
  }
}

export function getVoltageRtiOutput(args) {
  let decimalCode = utility.getDecNum(
    args.output_code.value,
    parseInt(
      args.parameterConfigData[args.selectedDevice].parameters.resolution,
    ),
    args.twos_complement.value,
  );

  return (
    (decimalCode *
      (isLsbSizeCalculated ? lsbSize : this.getLsbSizeValue(args))) /
    utility.getMultiplierForUnit(args.voltage_rti_output.unit.value)
  );
}

export function getInputRange(args) {
  let module = args.parameterConfigData[args.selectedDevice].calculationModule;
  let fsrValue;
  if (Object.keys(module).length > 0) {
    if (module.getFsrValue) {
      fsrValue = module.getFsrValue(
        args,
        args.parameterConfigData[args.selectedDevice].parameters,
        args.parameterConfigData[args.selectedDevice].controls.COMMON,
        args.parameterConfigData[args.selectedDevice].controls.codeToVoltage,
        args.parameterConfigData[args.selectedDevice].controls.voltageToCode,
      );
    }
  }
  return fsrValue / 2;
}

export function getInputVoltageRti(args) {
  return (
    parseFloat(args.voltage_rti.value) *
    utility.getMultiplierForUnit(args.voltage_rti.unit.value)
  );
}

export function getMaxCode(args) {
  return this.getNumOfCodes(args) / 2 - 1;
}

export function getMinCode(args) {
  return -(this.getNumOfCodes(args) / 2);
}

export function getNumOfCodes(args) {
  const bitResolution = parseInt(
    args.parameterConfigData[args.selectedDevice].parameters.resolution,
  );
  return 2 ** bitResolution;
}
export function getLsbSizeValue(args) {
  let module = args.parameterConfigData[args.selectedDevice].calculationModule;
  if (Object.keys(module).length > 0) {
    if (module.getLsbSize) {
      lsbSize = module.getLsbSize(
        args,
        args.parameterConfigData[args.selectedDevice].parameters,
        args.parameterConfigData[args.selectedDevice].controls.COMMON,
        args.parameterConfigData[args.selectedDevice].controls.codeToVoltage,
        args.parameterConfigData[args.selectedDevice].controls.voltageToCode,
      );
      isLsbSizeCalculated = true;
    }
  }
  return lsbSize;
}

export function getVoltageRangeStr(args) {
  const diffInputVoltage = this.getInputVoltageRti(args);
  const inputRange = this.getInputRange(args);
  if (
    diffInputVoltage >
    inputRange - (isLsbSizeCalculated ? lsbSize : this.getLsbSizeValue(args))
  ) {
    return "POSITIVE";
  } else if (diffInputVoltage < -inputRange) {
    return "NEGATIVE";
  }

  return "-";
}

export function isValidOutputCode(args) {
  let decimalCode = utility.getDecNum(
    args.output_code.value,
    parseInt(
      args.parameterConfigData[args.selectedDevice].parameters.resolution,
    ),
    args.twos_complement.value,
  );

  return (
    decimalCode >= this.getMinCode(args) && decimalCode <= this.getMaxCode(args)
  );
}
